<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:java="http://xml.apache.org/xslt/java"
extension-element-prefixes="java">

<xsl:include href="style.xsl"/>

<xsl:output method="html"/>  

<xsl:template match="page">
<xsl:variable name="collName" select="response/@from"/>
<html>
<xsl:call-template name="greenstoneHead"/>
<body xsl:use-attribute-sets="body-style">
<center>
<xsl:apply-templates select="response/collection|response/serviceCluster"><xsl:with-param name="collName" select="$collName"/></xsl:apply-templates>
</center>
<xsl:call-template name="greenstoneFooter"/>
</body>
</html>
</xsl:template>

<xsl:template match="collection">
<xsl:param name="collName">x</xsl:param>
<xsl:call-template name="collectionPageBanner">
<xsl:with-param name="collName"><xsl:value-of select="$collName"/></xsl:with-param>
<xsl:with-param name="collIcon"><xsl:value-of select="metadataList/metadata[@name='iconCollection']"/></xsl:with-param>
</xsl:call-template>
<p/><xsl:apply-templates select="metadataList">
<xsl:with-param name="collName"><xsl:value-of select="$collName"/></xsl:with-param>
</xsl:apply-templates>
<p/><xsl:apply-templates select="serviceList" mode="collection">
<xsl:with-param name="collName"><xsl:value-of select="$collName"/></xsl:with-param>
</xsl:apply-templates>
</xsl:template>

<xsl:template match="serviceCluster">
<xsl:param name="collName">x</xsl:param>
<xsl:call-template name="clusterPageBanner">
<xsl:with-param name="clusterName"><xsl:value-of select="$collName"/></xsl:with-param></xsl:call-template>
<p/><xsl:apply-templates select="metadataList">
<xsl:with-param name="collName"><xsl:value-of select="$collName"/></xsl:with-param>
</xsl:apply-templates>
<p/><xsl:apply-templates select="serviceList" mode="cluster">
<xsl:with-param name="collName"><xsl:value-of select="$collName"/></xsl:with-param>
</xsl:apply-templates>
</xsl:template>



<xsl:template match="metadataList">
<xsl:param name="collName">coll-name</xsl:param>
<h2><xsl:call-template name="text"><xsl:with-param name="key">about.description</xsl:with-param></xsl:call-template></h2>
<p/>
<table>
<xsl:for-each select="metadata">
<tr><td><b><xsl:value-of select="@name"/></b></td><td><xsl:value-of select="."/></td></tr>
</xsl:for-each>
</table>
</xsl:template>

<xsl:template match="serviceList" mode="collection">
<xsl:param name="collName">coll-name</xsl:param>
<xsl:variable name="library" select="ancestor::page/config/library_name"/>
<h2><xsl:call-template name="text"><xsl:with-param name="key">about.services</xsl:with-param></xsl:call-template></h2>
<table>
<xsl:for-each select="service[@type='query']">
<xsl:variable name="serviceName"><xsl:value-of select="@name"/></xsl:variable>
<tr>
<xsl:if test="contains($serviceName, 'Query')">
<xsl:variable name="orig-type"><xsl:value-of select="substring-before($serviceName, 'Query')"/></xsl:variable>
<xsl:variable name="lower-type" select="java:org.greenstone.gsdl3.util.XSLTUtil.toLower($orig-type)"/>
<td><a href="{$library}?a=q&amp;sa={$orig-type}&amp;c={$collName}"><xsl:value-of select="$serviceName"/></a></td>
</xsl:if>
<xsl:if test="contains($serviceName, 'Browse')">
<xsl:variable name="orig-type"><xsl:value-of select="substring-before($serviceName, 'Browse')"/></xsl:variable>
<xsl:variable name="lower-type" select="java:org.greenstone.gsdl3.util.XSLTUtil.toLower($orig-type)"/>
<td><a href="{$library}?a=b&amp;sa={$orig-type}&amp;c={$collName}"><xsl:value-of select="$serviceName"/></a></td>
</xsl:if>
<xsl:if test="contains($serviceName, 'Applet')">
<xsl:variable name="orig-type"><xsl:value-of select="substring-before($serviceName, 'Applet')"/></xsl:variable>
<td><a href="{$library}?a=a&amp;sa=d&amp;sn={$orig-type}&amp;c={$collName}"><xsl:value-of select="$serviceName"/></a></td>
</xsl:if>
</tr>
</xsl:for-each>
</table>
<xsl:call-template name="greenBar"/>
</xsl:template>


<xsl:template match="serviceList" mode="cluster">
<xsl:param name="collName">coll-name</xsl:param>
<xsl:variable name="library" select="ancestor::page/config/library_name"/>
<h2><xsl:call-template name="text"><xsl:with-param name="key">about.services</xsl:with-param></xsl:call-template></h2>
<table>
<xsl:for-each select="service">
<tr><td><a href="{$library}?a={$collName}&amp;sa={@name}"><xsl:value-of select="@name"/></a></td></tr>
</xsl:for-each>
</table>
<xsl:call-template name="greenBar"/>
</xsl:template>

</xsl:stylesheet>  

