/*
 *    PerlBuildServices.java 
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;

import org.greenstone.gsdl3.util.*;

import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.io.File;
import java.util.Locale;

/**
 * A Services class for building collections
 * provides a wrapper around the old perl scripts
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3567 $
 */
public class PerlBuildServices 
    extends ServicesImpl {

    // services offered
    private static final String NEW_SERVICE = "NewCollection";
    private static final String IMPORT_SERVICE = "ImportCollection";
    private static final String BUILD_SERVICE = "BuildCollection";
    private static final String ACTIVATE_SERVICE = "ActivateCollection";
    private static final String DELETE_SERVICE = "DeleteCollection";
    private static final String RELOAD_SERVICE = "ReloadCollection";

    // params used
    private static final String COL_PARAM = "collection";
    private static final String NEW_COL_NAME_PARAM = "collectionName";
    private static final String CREATOR_PARAM = "creator";

    // the list of the collections - store between some method calls
    private String[] collection_list_ = null;

    /** returns the service list for the subclass */
    protected Element getServiceList(String lang) {
	// for now, it is static and there is no lang stuff
	return short_service_info_;
    }

    /** returns a specific service description */
    protected Element getServiceDescription(String service, String lang) {
	Element descript = (Element)service_info_map_.get(service);
	// need to add in collection param list
	if (!service.equals(NEW_SERVICE)) {
	    Element param_list = (Element) GSXML.getChildByTagName(descript, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	    Element old_param = GSXML.getNamedElement(param_list, GSXML.PARAM_ELEM, GSXML.NAME_ATT, COL_PARAM);
	    collection_list_ = getCollectionList();
	    Element param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, null, collection_list_);
	    // put at front of the list
	    param_list.replaceChild(param, old_param);
	}
	// for now, create it on the fly - look at caching it later
	Element display = createServiceDisplay(service, lang);
	addServiceDisplay(descript, display);
	return descript;
	
    }

    /** creates a display element containing all the text strings needed to display the service page, in the language specified */
    protected Element createServiceDisplay(String service, String lang) {
	Element display = doc_.createElement(GSXML.DISPLAY_ELEM);
	display.appendChild(GSXML.createTextElement(doc_, GSXML.DISPLAY_NAME_ELEM,  getTextString(service+".name", lang)));
	display.appendChild(GSXML.createTextElement(doc_,  GSXML.DISPLAY_SUBMIT_ELEM, getTextString(service+".submit", lang)));

	Element param;
	if (service.equals(NEW_SERVICE)) {
	    // have the coll name and creator params
	    param = GSXML.createParameterDisplay(doc_, NEW_COL_NAME_PARAM, getTextString("param."+NEW_COL_NAME_PARAM, lang), null, null);
	    display.appendChild(param);
	    param = GSXML.createParameterDisplay(doc_, CREATOR_PARAM, getTextString("param."+CREATOR_PARAM, lang), null, null);
	    display.appendChild(param);
	} else {
	    // have just the coll param
	    //use the colleciton list already created - this method is only called directly after getServiceDescription()
	    param = GSXML.createParameterDisplay(doc_, COL_PARAM, getTextString("param."+COL_PARAM, lang), collection_list_, collection_list_);
	    display.appendChild(param);
	}	    

	return display;
	
    }

     /** passes the request Element to the appropriate service function*/
    protected Element processService(String name, Element request) {

	String language = request.getAttribute(GSXML.LANG_ATT);
	// extract the collection out of here
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(param_list);
	String coll_name = (String)params.get(COL_PARAM);
	
	Element response = doc_.createElement(GSXML.RESPONSE_ELEM);
	response.setAttribute(GSXML.FROM_ATT, name);
	Element status=null;
	if (name.equals(NEW_SERVICE)) {
	    status = newCollection(coll_name, language);
	} else if (name.equals(IMPORT_SERVICE)) {
	    status = importCollection(coll_name, language);
	} else if (name.equals(BUILD_SERVICE)) {
	    status = buildCollection(coll_name, language);
	} else if (name.equals(ACTIVATE_SERVICE)) {
	    status = activateCollection(coll_name, language);
	} else if (name.equals(DELETE_SERVICE)) {
	    status = deleteCollection(coll_name, language);
	} else if (name.equals(RELOAD_SERVICE)) {
	    status = reloadCollection(coll_name, language);
	} 
	
	if (status !=null) {
	    response.appendChild(status);
	    return response;
	}
	
	System.err.println("PerlBuildServices:should never get here. service type wrong:"+name);
	return null;
	    
    }

    /** configure the service module
     * for now, all services have type=build - need to think about this
     */
    public boolean configure(Element info) {
	System.out.println("configuring PerlBuildServices");

	Element e = null;
	// hard code in the services for now

	// set up short_service_info_ - for now just has name and type

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, NEW_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, IMPORT_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, BUILD_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, ACTIVATE_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, DELETE_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, RELOAD_SERVICE);
	short_service_info_.appendChild(e);

	//e = doc_.createElement(GSXML.SERVICE_ELEM);
	//e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	//e.setAttribute(GSXML.NAME_ATT, "AddDocument");
	//short_service_info_.appendChild(e);

	// set up service_info_map_ - for now, just has the same elements as above

	Element param_list; 
	Element param;

	String lang = Locale.US.toString();
	
	// should have full details about each service incl params lists etc.
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, NEW_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, NEW_COL_NAME_PARAM, GSXML.PARAM_TYPE_STRING, "", null);
	param_list.appendChild(param);
	param = GSXML.createParameter(doc_, CREATOR_PARAM, GSXML.PARAM_TYPE_STRING, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(NEW_SERVICE, e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, IMPORT_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(IMPORT_SERVICE, e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, BUILD_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(BUILD_SERVICE, e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, ACTIVATE_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(ACTIVATE_SERVICE, e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, DELETE_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(DELETE_SERVICE, e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_BUILD);
	e.setAttribute(GSXML.NAME_ATT, RELOAD_SERVICE);
	param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	param = GSXML.createParameter(doc_, COL_PARAM, GSXML.PARAM_TYPE_ENUM_SINGLE, "", null);
	param_list.appendChild(param);
	e.appendChild(param_list);
	service_info_map_.put(RELOAD_SERVICE, e);


	return true;
    }

    /** creates a new collection using mkcol.pl */
    protected Element newCollection(String coll_name, String lang) {
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	Text t = doc_.createTextNode("trying to create new coll:"+coll_name);
	status.appendChild(t);
	return status;
    }   
    
    /** imports a collection using import.pl */
    protected Element importCollection(String coll_name, String lang) {
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	Text t = doc_.createTextNode("trying to import coll:"+coll_name);
	status.appendChild(t);
	return status;
    }   
    /** builds a collection using buildcol.pl */
    protected Element buildCollection(String coll_name, String lang) {
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	Text t = doc_.createTextNode("trying to build coll:"+coll_name);
	status.appendChild(t);
	return status;
    }   
    protected Element reloadCollection(String coll_name, String lang) {
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	
	Element message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element request = doc_.createElement(GSXML.REQUEST_ELEM);
	request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_CONFIGURE);
	request.setAttribute(GSXML.TO_ATT, "");
	message.appendChild(request);
	Element command = doc_.createElement(GSXML.CONFIGURE_ELEM);
	command.setAttribute(GSXML.TYPE_ATT, GSXML.COLLECTION_ELEM);
	command.setAttribute(GSXML.NAME_ATT, coll_name);
	command.setAttribute(GSXML.ACTION_ATT, GSXML.CONFIG_ACTION_ACTIVATE);
	request.appendChild(command);
	Element response = router_.process(message); // at the moment, get no info in response so ignore it

	String[] args={coll_name};

	if (response == null) {
	    Text t = doc_.createTextNode(getTextString("reload.activate_error", args , lang));
	    status.appendChild(t);
	    return status;
	}
	// if we got here, we have succeeded!
	Text t = doc_.createTextNode(getTextString("reload.success", args, lang));
	status.appendChild(t);
	return status;	
    }   

    /** activates a collection by removing index, copying building over 
     * also instructs the MR to activate this new/modified collection
     * fails if no valid building directory */ 
    protected Element activateCollection(String coll_name, String lang) {
	// the return element
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	String[] args={coll_name};

	// check that we have buildcfg.xml in the building directory
	File coll_build_dir = new File(GSFile.collectionBuildDir(site_home_, coll_name));
	if (!coll_build_dir.exists()) {
	    Text t = doc_.createTextNode(getTextString("activate.build_dir_error", args, lang));
	    status.appendChild(t);
	    return status;
	}
	File coll_build_file = new File(GSFile.collectionBuildConfigFileBuilding(site_home_, coll_name));
	if (!coll_build_file.exists()) {
	    Text t = doc_.createTextNode(getTextString("activate.build_config_error", args, lang));
	    status.appendChild(t);
	    return status;
	}
	
	
	File coll_index_dir = new File(GSFile.collectionIndexDir(site_home_, coll_name));
	if (coll_index_dir.exists()) {
	    // remove it
	    if (!GSFile.deleteFile(coll_index_dir)) {
		Text t = doc_.createTextNode(getTextString("activate.rm_index_error", args, lang));
		status.appendChild(t);
		return status;
	    }
	}
	
	// copy building to index
	if (!GSFile.moveDirectory(coll_build_dir, coll_index_dir)) {
	    Text t = doc_.createTextNode(getTextString("activate.rename_building_error", args, lang));
	    status.appendChild(t);
	    return status;
	}
	
	// now have to tell the message router
	return reloadCollection(coll_name, lang);

    }
    /** deletes a collection - removes the whole directory
     * also instructs the MR to deactivate this  collection*/ 
    protected Element deleteCollection(String coll_name, String lang) {
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	
	String[] args={coll_name};
	File coll_dir = new File(GSFile.collectionBaseDir(site_home_, coll_name));
	if (coll_dir.exists()) {
	    if (!GSFile.deleteFile(coll_dir)) {
		Text t = doc_.createTextNode(getTextString("delete.delete_error", args, lang));
		status.appendChild(t);
		return status;
	    }
	    // now tell the MR about it

	    Element message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element request = doc_.createElement(GSXML.REQUEST_ELEM);
	    request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_CONFIGURE);
	    request.setAttribute(GSXML.TO_ATT, "");
	    message.appendChild(request);
	    Element command = doc_.createElement(GSXML.CONFIGURE_ELEM);
	    command.setAttribute(GSXML.TYPE_ATT, GSXML.COLLECTION_ELEM);
	    command.setAttribute(GSXML.NAME_ATT, coll_name);
	    command.setAttribute(GSXML.ACTION_ATT, GSXML.CONFIG_ACTION_DEACTIVATE);
	    request.appendChild(command);
	    
	    Element response = router_.process(message); // at the moment, get no info in response so ignore it
	    if (response == null) {
		Text t = doc_.createTextNode(getTextString("delete.deactivate_error", args, lang));
		status.appendChild(t);
		return status;
	    }
	}
	// success   
       
	Text t = doc_.createTextNode(getTextString("delete.success", args, lang));
	status.appendChild(t);
	return status;
    }

    protected String[] getCollectionList() {
	
	File collectDir = new File(GSFile.collectDir(site_home_));
	if (!collectDir.exists()) {
	    System.err.println("couldn't find collect dir: "+collectDir.toString());
	    return null;
	}
	System.out.println("PerlBuildServices: reading thru directory "+collectDir.getPath()+" to find collections.");
	File[] contents = collectDir.listFiles();
	int num_colls=0;
	for (int i=0; i<contents.length;i++) {
	    if(contents[i].isDirectory() && !contents[i].getName().startsWith("CVS")) {
		num_colls ++;
	    }
	}
	
	String[] names= new String[num_colls];
	
	for (int i=0, j=0; i<contents.length;i++) {
	    if(contents[i].isDirectory()) {
		String colName = contents[i].getName();
		if (!colName.startsWith("CVS")) {
		    names[j] = colName;
		    j++;
		}
		
	    } 
	}
	
	return names;
	
    }
   

}
